// ********************************************************************
// LC-Display DIP204 - Grundfunktionen 
// LCD (DIP204-4 4x20) 8-Bit Interface mit KS0073 Controller
// AVR-Version / GGC 4.3.2
// Stand: 01.11.2009
// Author: Matthias Kahnt
// ********************************************************************
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// der Authoren erforderlich
// Kontakt: Matthias Kahnt, e-mail: az51@gmx.net
// ********************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// ********************************************************************
//
//
// ***********************************************************************
// Schreiben eines Zeichens an das LCD-Modul
// bergabe: lcd_byte : Auszugebendes Zeichen/Steuerzeichen
//           lcd_mode : 0 - Daten
//                      1 - Steuerzeichen
// ***********************************************************************
void write_lcd(unsigned char lcd_byte, unsigned char lcd_mode) {
unsigned char lcd_busy = 1;
  // Testen ob LCD-Modul noch im internen Zyklus ist
  LCD_RW_H;
  LCD_RS_L;
  LCD_PORT = 0x00;     // nur bei AVR: Port A alle PA0 ... PA7  als Eingang
  // LCD_DATA = 0xFF;  // nur bei 8051: Datenportlatch fr Rcklesen vorbereiten (8051-Portspezifik) 
  while(lcd_busy) {
    LCD_E1_H;
    _delay_us(5);     // Impulsverzgerung  
    if (!(LCD_DATA_READ & 0x80)) lcd_busy = 0; // BUSY-Flag abfragen
    LCD_E1_L;
    _delay_us(5);     // Impulsverzgerung      
  }
  LCD_PORT = 0xFF;    // nur bei AVR: Port A alle PA0 ... PA7  als Ausgang
  // LCD-Anzeige ist zum Empfang neuer Zeichen bereit
  LCD_DATA = lcd_byte; // Byte ausgeben
  LCD_RW_L;
  if (lcd_mode) LCD_RS_L; else LCD_RS_H;
  LCD_E1_H;
  _delay_us(30);     // Impulsverzgerung
  LCD_E1_L;
} 

// ***********************************************************************
// Lesen eines Zeichens von dem LCD-Modul
// Rckgabe: lcd_byte : gelesenes Zeichen
// ***********************************************************************
unsigned char read_lcd(void) {
unsigned char lcd_busy = 1;
unsigned char lcd_byte;
  // Testen ob LCD-Modul noch im internen Zyklus ist
  LCD_RW_H;
  LCD_RS_L;
  LCD_PORT = 0x00;     // nur bei AVR: Port A alle PA0 ... PA7  als Eingang
  // LCD_DATA = 0xFF;  // nur bei 8051: Datenportlatch fr Rcklesen vorbereiten (8051-Portspezifik) 
  while(lcd_busy) {
    LCD_E1_H;
    _delay_us(5);     // Impulsverzgerung  
    if (!(LCD_DATA_READ & 0x80)) lcd_busy = 0; // BUSY-Flag abfragen
    LCD_E1_L; // = 0;
    _delay_us(5);     // Impulsverzgerung      
  }
  // LCD-Anzeige ist bereit
  LCD_RW_H;
  LCD_RS_H;
  LCD_E1_H;
  _delay_us(30);            // Impulsverzgerung
  lcd_byte = LCD_DATA_READ; // Byte einlesen
  LCD_E1_L;
  LCD_PORT = 0xFF;          // nur bei AVR: Port A alle PA0 ... PA7  als Ausgang
  return lcd_byte;
} 

// ***********************************************************************
// Lschen des LCD-Display
// ***********************************************************************
void clear_lcd(void) {
  write_lcd(0x01,1);        // Diplay lschen
  lcd.x = lcd.y = 0;
}

// ***********************************************************************
// Ausgabe eines ASCII-Zeichen positioniert auf dem LCD-Modul
// bergabe: lcd_x : Spalte (0...SPALTEN_MAX-1)
//           lcd_y : Zeile  (0...ZEILEN_MAX-1)
//           lcd_ascii : ASCII-Zeichen
// *********************************************************************** 
void write_byte_lcd(unsigned char lcd_x, unsigned char lcd_y, unsigned char lcd_ascii) {
unsigned char lcd_offset = 0;
  if (lcd_x > (SPALTEN_MAX - 1)) lcd_x = 0;
  if (lcd_y > (ZEILEN_MAX  - 1)) lcd_y = 0;
  switch (lcd_y) {
    case 0:  lcd_offset = 0x80; break;  // Zeile 1
    case 1:  lcd_offset = 0xA0; break;  // Zeile 2 
    case 2:  lcd_offset = 0xC0; break;  // Zeile 3
    case 3:  lcd_offset = 0xE0; break;  // Zeile 4 
    default: lcd_offset = 0x80;         // Zeile>4
  };
  write_lcd(lcd_x+lcd_offset,1);        // Kursorposition setzen
  write_lcd(lcd_ascii,0);               // Ausgabe des ASCII-Zeichen an der Kursorposition
}

// ***********************************************************************
// Ausgabe einer Zeichenkette positioniert auf dem LCD-Modul
// bergabe: lcd_x : Spalte (0...SPALTEN_MAX-1)
//           lcd_y : Zeile  (0...ZEILEN_MAX-1)
//           lcd_zeichen : Adresse der auszugebenden format.  Zeichenkette
//           clr_line    : Lschen bis Zeilenende
//                         1 - Lschen
//                         0 - kein Lschen
// ***********************************************************************
void printf_lcd(unsigned char lcd_x, unsigned char lcd_y, char *lcd_zeichen, unsigned char clr_line) {
unsigned char lcd_i;
unsigned char lcd_offset = 0;
  if (lcd_x > (SPALTEN_MAX - 1)) lcd_x = 0;
  if (lcd_y > (ZEILEN_MAX  - 1)) lcd_y = 0;
  switch (lcd_y) {
    case 0:  lcd_offset = 0x80; break;  // Zeile 1
    case 1:  lcd_offset = 0xA0; break;  // Zeile 2 
    case 2:  lcd_offset = 0xC0; break;  // Zeile 3
    case 3:  lcd_offset = 0xE0; break;  // Zeile 4 
    default: lcd_offset = 0x80;         // Zeile>4
  }
  write_lcd(lcd_x+lcd_offset,1);        // Kursorposition setzen
  // Ausgabe der Zeichenkette ab der Kursorposition
  lcd_offset = strlen(lcd_zeichen);     // Lnge der Zeichenkette
  if (lcd_offset > SPALTEN_MAX) lcd_offset = SPALTEN_MAX;
  for(lcd_i = lcd_offset ; lcd_i ; lcd_i--) {
    write_lcd(*lcd_zeichen,0);
    lcd_zeichen++;
  }
  if (clr_line) {
    // Lschen bis Zeilenende
    for(lcd_i = SPALTEN_MAX - lcd_offset - lcd_x ; lcd_i ; lcd_i--) write_lcd(' ',0);
  }
}

// ***********************************************************************
// Blinkposition auf dem LCD-Displays setzen / Blinken ein/ausschalten
// bergabe: lcd_x : Spalte (0...SPALTEN_MAX-1)
//           lcd_y : Zeile  (0...ZEILEN_MAX-1)
//           lcd_blink :   0 - Blinken/Cursor aus
//                         1 - Blinken an
//                         2 - Cursor  an
// ***********************************************************************
void blink_lcd(unsigned char lcd_x, unsigned char lcd_y, unsigned char lcd_blink) {
unsigned char lcd_offset;
  write_lcd(0x0C,1);                     // KURSOR ausschalten
  if (lcd_x > (SPALTEN_MAX - 1)) lcd_x = 0;
  if (lcd_y > (ZEILEN_MAX  - 1)) lcd_y = 0;
  switch (lcd_y) {
    case 0:  lcd_offset = 0x80; break;   // Zeile 1
    case 1:  lcd_offset = 0xA0; break;   // Zeile 2 
    case 2:  lcd_offset = 0xC0; break;   // Zeile 3
    case 3:  lcd_offset = 0xE0; break;   // Zeile 4 
    default: lcd_offset = 0x80;          // Zeile>4
  };
  write_lcd(lcd_x+lcd_offset,1);         // Blinkposition setzen
  if (lcd_blink == 1) write_lcd(0x0D,1); // Blinken ein
  if (lcd_blink == 2) write_lcd(0x0E,1); // Cursor ein
}

// ***********************************************************************
// LCD Monitorfunktion (einzelnes Zeichen)
// Ausgabe des Zeichen an aktueller Kursorposition, wenn LCD voll ist
// scollen alle Zeilen nach oben, Ausgabe dann in letzter Zeile.
// bergabe : lcd_byte als ASCII
// *********************************************************************** 
void lcd_monitor_byte(unsigned char lcd_byte) {
unsigned char lcd_offset;
unsigned char temp;
  // Umkodierung von ASCII und Steuerzeichen (LCD-Spezifik)
  switch (lcd_byte) {
    case 0x0A: lcd.x = 0; return; // Rcklauf
    case 0x0D: lcd.x = 0; lcd.y++; return; // Neue Zeile
    case 0x40: lcd_byte = 0xA0; break; // at @
    case 0x5B: lcd_byte = 0xFA; break; // Klammer [
    case 0x5C: lcd_byte = 0xFB; break; // Backslash 
    case 0x5D: lcd_byte = 0xFC; break; // Klammer ]
    case 0x5E: lcd_byte = 0x1D; break; // Hoch ^
    case 0x5F: lcd_byte = 0xC4; break; // Unterstrich _
    case 0x7B: lcd_byte = 0xFD; break; // Klammer {
    case 0x7C: lcd_byte = 0xFE; break; // Strich |
    case 0x7D: lcd_byte = 0xFF; break; // Klammer }
    case 0x7E: lcd_byte = 0xCE; break; // Tilde ~
    case 0xB5: lcd_byte = 0x8F; break; // 
    case 0xC4: lcd_byte = 0x5B; break; // 
    case 0xD6: lcd_byte = 0x5C; break; // 
    case 0xDC: lcd_byte = 0x5E; break; // 
    case 0xE4: lcd_byte = 0x7B; break; // 
    case 0xF6: lcd_byte = 0x7C; break; // 
    case 0xFC: lcd_byte = 0x7E; break; // 
    case 0xDF: lcd_byte = 0xBE; break; // 
  }
  // Prfung auf Zeilenwechsel
  if (lcd.x > (SPALTEN_MAX - 1)) {
    lcd.x = 0;
    lcd.y++;
  }
  // Test auf Zeilenscolling
  if (lcd.y > (ZEILEN_MAX - 1)) {
    // Scrollen erforderlich
    _delay_ms(SCOLL_DELAY);       // Pause, vor Scrollfunktion
    lcd_offset = 0x80;  // oberste Zeile adressieren
    for (lcd.y = 0; lcd.y < (ZEILEN_MAX - 1); lcd.y++) {
      for (lcd.x = 0; lcd.x < SPALTEN_MAX; lcd.x++) {
        write_lcd(lcd.x + lcd_offset + 0x20,1); // Cursor Quellzeile
        temp = read_lcd();                      // Zeichen auslesen
        write_lcd(lcd.x + lcd_offset,1);        // Cursor Zielzeile
        write_lcd(temp,0);                      // Zeichen schreiben
      }
      lcd_offset += 0x20;                 // nchste Zeile adressieren
    }
    printf_lcd(0,(ZEILEN_MAX - 1)," ",1); // unterste Zeile lschen
    // sprintf(&lcd_buffer[0],"%s"," "); // Einschaltmeldung  
    // printf_lcd(0,(ZEILEN_MAX - 1),lcd_buffer,1);
	lcd.y = (ZEILEN_MAX - 1);             // Weiter Ausgaben nun in letzter Zeile
    lcd.x = 0;
  }
  // Ausgabezeile adressieren (Bit 7 immer 1)
  switch (lcd.y) {
    case 0:  lcd_offset = 0x80; break;  // Zeile 1
    case 1:  lcd_offset = 0xA0; break;  // Zeile 2 
    case 2:  lcd_offset = 0xC0; break;  // Zeile 3
    case 3:  lcd_offset = 0xE0; break;  // Zeile 4 
    default: lcd_offset = 0x80;         // Zeile>4
  };
  write_lcd(lcd.x + lcd_offset,1);      // Kursorposition setzen
  write_lcd(lcd_byte,0);                // Ausgabe des ASCII-Zeichen an der Kursorposition
  lcd.x++;                              // Position fr nchstes Zeichen erhhen
}

// ***********************************************************************
// LCD Monitorfunktion (String)
// bergabe : lcd_zeichen (Adresse)
// *********************************************************************** 
void lcd_monitor_string(char *lcd_zeichen) {
unsigned char lcd_i;
  // Ausgabe der Zeichenkette ab der Kursorposition
  for(lcd_i = strlen(lcd_zeichen); lcd_i; lcd_i--) {
    lcd_monitor_byte(*lcd_zeichen);
    lcd_zeichen++;
  }
  lcd_monitor_byte(0x0D); // Neue Zeile
}

// ***********************************************************************
// Grundinitialisierung des LCD-Moduls
// ***********************************************************************
void init_lcd(void) {
  LCD_E1_L;                 // LCD disable
  _delay_ms(100);           // Wartezeit nach POWER-ON bis VCC stabil ist
  // Grundinitialisierung
  write_lcd(0x34,1);        // 34 0011 0100 (8Bit Bus RE=1)
  write_lcd(0x09,1);        // 09 0000 1001 (4 Zeilen Mode)
  write_lcd(0x30,1);        // 30 0011 0000 (8Bit Bus RE=0) 
  write_lcd(0x0C,1);        // 0000 1111 (Dipl. ein, Cursor aus, Cursor blinken aus)
  write_lcd(0x01,1);        // 0000 0001 (Diplay lschen, Cursor auf Pos. 1)
  write_lcd(0x06,1);        // 0000 0110 (Autoinkrement)
}
