// ********************************************************************
// VNC1L/VDIP1 Vinculum - Grundfunktionen 
// AVR-Version / GGC 4.3.2
// Stand: 01.11.2009
// Author: Matthias Kahnt
// ********************************************************************
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// der Authoren erforderlich
// Kontakt: Matthias Kahnt, e-mail: az51@gmx.net
// ********************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// ********************************************************************
//
//
// ***********************************************************************
// VNC1L - USB HOST CONTROLLER - SPI-MODE (MC: Master, VNC1L: SLAVE)
// VDIP1: Jumper-Set: J3 - Pulldown / J4 - Pullup
//        VDAP Firmware muss installiert sein, mit Version 3.68 getestet
// ***********************************************************************

// ***********************************************************************
// VNC1L - Intialisierung SPI-Interface und VNC1L
// SPI Mode
// ***********************************************************************
void vnc_init(void) {
  CS_VNC_L;  // SPI-Interface inaktiv
  SCLK_L;    // Ruhezustand der SPI-Leitungen herstellen
  SDI_H;
  RE_VNC_L;  // Hardware-Reset VNC1L
  _delay_ms(100); // Pause
  RE_VNC_H;  // VNC1L starten
}

// ***********************************************************************
// VNC1L - Senden eines Zeichens
// SPI Mode
// bergabe: wr_byte   : Auszugebendes Zeichen
// Rckgabe: statusbit : 0=Accept (ok) 1=Reject (nicht ok)
// ***********************************************************************
unsigned char vnc_wr_byte(unsigned char wr_byte) {
unsigned char stelle, status;
  CS_VNC_H;        // SPI-Interface aktiv
  SDI_H;           // Startbit (immer 1)
  SCLK_H;  SCLK_L; // L/H Flanke
  SDI_L;           // RW-Bit (0=Write)
  SCLK_H;  SCLK_L; // L/H Flanke
  SDI_L;           // ADDR (0=Daten)
  SCLK_H;  SCLK_L; // L/H Flanke
  // 8 Datenbits senden (MSB zuerst)
  for ( stelle = 0x80; stelle; stelle >>= 1 ) {
    if (wr_byte & stelle) SDI_H; else SDI_L;
    SCLK_H;  SCLK_L; // L/H Flanke
  }
  // Statusbit einlesen
  status = SDO_VNC; // Statusbit (0=Accept, 1=Reject)
  if (status) {
	vnc.error = 4;  // Fehler Reject (wr_byte wurde nicht akzeptiert)!!!
	LED_D1_EIN;
  }
  SCLK_H;  SCLK_L; // L/H Flanke
  // End Sequenz
  CS_VNC_L;        // SPI-Interface inaktiv
  SCLK_H;  SCLK_L; // L/H Flanke
  // SDO    = 1;   // nur bei 8051: Datenlatch freigeben (8051-Spezifik)  
#ifdef LCD_MONITORING
  lcd_monitor_byte(wr_byte); // gesendetes Zeichen auf LCD anzeigen
#endif

#ifdef  USART_MONITORING
  usart_write_char(wr_byte);
#endif
  _delay_us(10);
  return status;
} 

// ***********************************************************************
// VNC1L - Empfangen eines Zeichens
// SPI Mode
// bergabe: rd_fkt    : Funktion (0=Datenbyte, 1=Statusbyte)
// Rckgabe: Datenbyte/Statusbit ber vnc-Stukturvariable
// ***********************************************************************
void vnc_rd_byte(unsigned char rd_fkt) {
unsigned char stelle, daten;
  daten  = 0;
  CS_VNC_H;        // SPI-Interface aktiv
  SDI_H;           // Startbit (immer 1)
  SCLK_H;  SCLK_L; // L/H Flanke
  SDI_H;           // RW-Bit (1=Read)
  SCLK_H;  SCLK_L; // L/H Flanke
  if (rd_fkt) SDI_H; else SDI_L; // ADDR (0=Daten, 1=SPI-Statusinformation) 
  SCLK_H;  SCLK_L; // L/H Flanke
  // 8 Datenbits empfangen (MSB zuerst)
  for ( stelle = 0x80; stelle; stelle >>= 1 ) {
    if (SDO_VNC) daten |= stelle;
    SCLK_H;  SCLK_L; // L/H Flanke
  }
  // Statusbit einlesen
  vnc.status = SDO_VNC; // Statusbit (0=New Data, 1=Old Data)
  SCLK_H;  SCLK_L; // L/H Flanke
  // End Sequenz
  CS_VNC_L;        // SPI-Interface inaktiv
  SCLK_H;  SCLK_L; // L/H Flanke
  // SDO    = 1;   // nur bei 8051: Datenlatch freigeben (8051-Spezifik)  
  if (!vnc.status) {
    // empfangenes Zeichen ist ein neues Zeichen
    // Empfangsbuffer um eine Stelle verschieben
    for ( stelle = 9; stelle; stelle-- ) vnc.rec_buffer[stelle] = vnc.rec_buffer[stelle - 1];
    vnc.rec_buffer[0] = daten; // neues Zeichen an 1. Stelle speichern
#ifdef LCD_MONITORING
    lcd_monitor_byte(vnc.rec_buffer[0]); // empfangenes Zeichen auf LCD anzeigen
#endif

#ifdef  USART_MONITORING
  usart_write_char(vnc.rec_buffer[0]);
#endif

  }
  return;
} 

// ***********************************************************************
// VNC1L - Senden eines Kommandos
// bergabe: wr_cmd : Kommando (Adresse)
// ***********************************************************************
void vnc_wr_cmd(char *wr_cmd) {
unsigned char vnc_i; // vnc_offset;
  for(vnc_i = strlen(wr_cmd); vnc_i; vnc_i--) {
    if (vnc_wr_byte(*wr_cmd)) {
	  vnc.error = 1;  // Fehler berlauf!!!
	  LED_D1_EIN;
    }
    wr_cmd++;
	_delay_ms(5);
  }
  _delay_ms(100); // Pause
  if (vnc_wr_byte(0x0D)) {
    vnc.error = 2;  // Fehler Endkennung!!!
	LED_D1_EIN;
  }
  _delay_ms(100); // Pause, um VNC1L Zeit zur Antwort zu geben
}

// ***********************************************************************
// VNC1L - Auf Antwortstring warten und analysieren
// bergabe: 1 = Prfung auf Prompt und Fehlermeldungen
//           2 = Prfung nur auf Prompt
// Rckgabe: 0 = Timeout/Fehler
//           1 = Prompt empfangen
//           2 = command failed empfangen
// ***********************************************************************
unsigned char vnc_rd_answer(unsigned char buffer_check) {
unsigned char antw = 0;
unsigned int rd_answer_versuche = 1000; // TIMEOUT bis Fehler/Abbruch ggf. ANPASSEN!
  while(rd_answer_versuche) {
    vnc_rd_byte(0); // Ein Zeichen per SPI von VNC1L empfangen
    switch (buffer_check) {
        case 1: // command failed
                if ((vnc.rec_buffer[2] == 'l') && (vnc.rec_buffer[1] == 'e') && (vnc.rec_buffer[0] == 'd')) antw = 2;
				// hier knnte auf weitere Fehlermeldungen gecheckt werden...
                // kein break; hier!!!
        case 2: // nur Prompt Check
                if ((vnc.rec_buffer[2] == 0x5C) && (vnc.rec_buffer[1] == '>') && (vnc.rec_buffer[0] == 0x0D)) {
                  // Test auf \> und CR war ok
                  antw = 1;
                  vnc.rec_buffer[0] = 0; // Buffer verwerfen (letzes Zeichen lschen reicht aus!)
                }
    }
    if (antw == 2) vnc_wr_byte(0x0D); // zustzliches CR um Fehler zu besttigen, dann kommt wieder ein Prompt (???)
    if (antw) break;
	rd_answer_versuche--;
	_delay_ms(20);
  }
  return antw;
}

// ***********************************************************************
// VNC1L - Auf PROMPT warten
// Bei Fehler Fehlerflag setzen
// ***********************************************************************
void vnc_prompt_check(void) {
unsigned char pr_check;
  pr_check = vnc_rd_answer(PROMPT_CHECK);
  if (!pr_check) {
    vnc.error = 3; // Fehler keim Prompt empfangen
	LED_D1_EIN;
  }
  return;
}

// ***********************************************************************
// VNC1L - Fehlermeldung
// ***********************************************************************
void vnc_fehlermeldung(void) {
  LED_D1_EIN;
#ifdef LCD_MONITORING
  sprintf (&lcd_buffer[0],"->Fehler-Nr.: %2d",vnc.error);
  //lcd_monitor_string("\nVDIP1/VNC1L-Fehler!"); // Fehlermeldung auf LCD anzeigen
  lcd_monitor_string(lcd_buffer); // Fehlermeldung auf LCD anzeigen
#endif
  // Hier Fehler auswerten
  // Zur Zeit ist hier das Programm zu Ende!
  while(1);
}


// ***********************************************************************
// VNC1L - Erzeugen eines Text-Files auf dem USB-Stick
// bergabe: filename   : Adresse des Filenamen
//           fileinhalt : Adresse des Textblockes
//           funktion   : File 0=Neu oder 1=Inhalt anhngen
// ***********************************************************************
void vnc_wr_txtfile(char *filename, char *fileinhalt, char funktion) {
unsigned int vnc_i;
unsigned int filelen, filecr;
char *fileptr;
unsigned char batch_wr = 0;
  // Byte-Anzahl berechnen
  fileptr = fileinhalt;
  filelen = 0;
  filecr  = 0;
  while (*fileptr != 0xFF) {
    if (*fileptr) filelen++; else filecr++;
    fileptr++;
  }
  // 1. Promptzeichen liegt bereits vor
  do {
    switch (batch_wr) { // Batchprozess
      case 0:  // File ffnen bzw. Erstellen
               sprintf (&vnc.send_buffer[0],"OPW %s",filename);
               vnc_wr_cmd(vnc.send_buffer);
               break;
      case 1:  // File Position bei Bedarf auf Anfang setzen
               if (!funktion) {
                 vnc_wr_cmd("SEK 0");
                 break;
               } else batch_wr++;  // Nun wird sofort auch noch case 2: ausgefhrt
      case 2:  // File Byteanzahl
               sprintf (&vnc.send_buffer[0],"WRF %d",filelen);
               vnc_wr_cmd(vnc.send_buffer);
               fileptr = fileinhalt;
               for(vnc_i = 0; vnc_i < (filelen + filecr); vnc_i++) {
                 if (*fileptr) vnc_wr_byte(*fileptr); // NULL-Terminierung ausblenden
                 fileptr++;
				 _delay_us(100); // nur bei hohen Taktfrequenzen erforderlich
               }
               break;
      case 3:  // File schlieen
               sprintf (&vnc.send_buffer[0],"CLF %s",filename);
               vnc_wr_cmd(vnc.send_buffer);
               break;
    } // Switch
    _delay_ms(200); // Pause
    batch_wr++;    // nchster Schritt
    if (batch_wr == 4) return; // Fertig
  } while (vnc_rd_answer(PROMPT_UND_ERROR_CHECK) == PROMPT_OK); // Prfung auf nchstes Promptzeichen >
}

// ***********************************************************************
// VNC1L - Einlesen eines Files vom dem USB-Stick in den Speicher
// bergabe: filename   : Adresse des Filenamen
//           fileinhalt : Adresse des Ziel-Datenspeicheres
//           filelen    : Anzahl der Bytes
// ***********************************************************************
void vnc_rd_file(char *filename, char *fileinhalt, unsigned int filelen) {
unsigned char batch_wr = 0;
  // 1. Promptzeichen liegt bereits vor
  do {
    switch (batch_wr) { // Batchprozess
      case 0:  // File fr lesen ffnen
               sprintf (&vnc.send_buffer[0],"OPR %s",filename);
               vnc_wr_cmd(vnc.send_buffer);
               break;
      case 1:  // File Position immer auf Anfang setzen (knnte man auch auf jede Pos. setzen!)
               vnc_wr_cmd("SEK 0");
               break;
      case 2:  // File Byteanzahl einlesen
               sprintf (&vnc.send_buffer[0],"RDF %d",filelen);
               vnc_wr_cmd(vnc.send_buffer);
               while (filelen) {
                 vnc_rd_byte(0);
                 if (!vnc.status) {
                   // neues Zeichen eingelesen (bisher keine Prfung auf Bufferberlauf!!!)
                   *fileinhalt = vnc.rec_buffer[0];
                   fileinhalt++;
                   filelen--;
                 }
				 _delay_us(100); // nur bei hohen Taktfrequenzen erforderlich
               }
               // Die gelesenen Zeichen stehten nun im Buffer und knnen ausgewertet werden.
               break;
      case 3:  // File schlieen
               sprintf (&vnc.send_buffer[0],"CLF %s",filename);
               vnc_wr_cmd(vnc.send_buffer);
               break;
    }
    _delay_ms(200); // Pause
    batch_wr++;    // nchster Schritt
    if (batch_wr == 4) return; // Fertig
  } while (vnc_rd_answer(PROMPT_UND_ERROR_CHECK) == PROMPT_OK); // Prfung auf nchstes Promptzeichen >
}

// ***********************************************************************
// VNC1L - Erzeugen eines Binr-Files auf dem USB-Stick
// bergabe: filename   : Adresse des Filenamen
//           fileinhalt : Adresse des Datenblockes
//           filelen    : Anzahl der Bytes
// ***********************************************************************
void vnc_wr_binfile(char *filename, char *fileinhalt, unsigned int filelen) {
unsigned int vnc_i;
unsigned char batch_wr = 0;
  // 1. Promptzeichen liegt bereits vor
  do {
    switch (batch_wr) { // Batchprozess
      case 0:  // File ffnen bzw. Erstellen
               sprintf (&vnc.send_buffer[0],"OPW %s",filename);
               vnc_wr_cmd(vnc.send_buffer);
               break;
      case 1:  // File Position immer auf Anfang setzen (knnte man auch auf jede Pos. setzen!)
               vnc_wr_cmd("SEK 0");
               break;
      case 2:  // File Byteanzahl
               sprintf (&vnc.send_buffer[0],"WRF %d",filelen);
               vnc_wr_cmd(vnc.send_buffer);
               for(vnc_i = 0; vnc_i < filelen; vnc_i++) {
                 vnc_wr_byte(*fileinhalt);
                 fileinhalt++;
				 _delay_us(100); // nur bei hohen Taktfrequenzen erforderlich
               }
               break;
      case 3:  // File schlieen
               sprintf (&vnc.send_buffer[0],"CLF %s",filename);
               vnc_wr_cmd(vnc.send_buffer);
               break;
    }
    _delay_ms(200); // Pause
    batch_wr++;    // nchster Schritt
    if (batch_wr == 4) return; // Fertig
  } while (vnc_rd_answer(PROMPT_UND_ERROR_CHECK) == PROMPT_OK); // Prfung auf nchstes Promptzeichen >
}
